package dev.fyloz.backup

import dev.fyloz.backup.data.FileProvider
import dev.fyloz.backup.data.LocalFileProvider
import dev.fyloz.backup.data.MongoDatabase
import dev.fyloz.backup.exceptions.HttpException
import dev.fyloz.backup.exceptions.NotFoundException
import dev.fyloz.backup.exceptions.ValidationException
import dev.fyloz.backup.logic.injection.LogicInjection
import dev.fyloz.backup.modules.backup.backupModule
import dev.fyloz.backup.repositories.injection.RepositoryInjection
import io.ktor.http.*
import io.ktor.serialization.kotlinx.json.*
import io.ktor.server.application.*
import io.ktor.server.netty.*
import io.ktor.server.plugins.callloging.*
import io.ktor.server.plugins.contentnegotiation.*
import io.ktor.server.plugins.statuspages.*
import io.ktor.server.response.*
import io.ktor.server.routing.*
import kotlinx.serialization.SerializationException
import org.koin.dsl.module
import org.koin.ktor.plugin.Koin
import org.koin.logger.slf4jLogger
import org.slf4j.event.Level

// https://github.com/mathias21/KtorEasy/tree/master

fun main(args: Array<String>) = EngineMain.main(args)

fun Application.module() {
    install(CallLogging) {
        level = Level.DEBUG
    }
    install(ContentNegotiation) {
        json()
    }

    module {
        install(Koin) {
            slf4jLogger()
            modules(
                RepositoryInjection.koinBeans,
                LogicInjection.koinBeans,

                module {
                    single { MongoDatabase() }
                    single<FileProvider> { LocalFileProvider() }
                }
            )
        }
    }

    install(Routing) {
        route("/v1") {
            backupModule()
        }
    }

    install(StatusPages) {
        exception<HttpException> { call, cause ->
            val statusCode = when (cause) {
                is ValidationException -> HttpStatusCode.BadRequest
                is NotFoundException -> HttpStatusCode.NotFound
            }

            call.respondText(cause.message, status = statusCode)
        }

        exception<SerializationException> { call, cause ->
            call.respondText(cause.localizedMessage, status = HttpStatusCode.BadRequest)
        }

        exception<Throwable> { call, cause ->
            call.respondText(text = "500: $cause", status = HttpStatusCode.InternalServerError)
        }
    }
}

// Backups:
// Backups will be incremental for x time
// After, the older backups will be put in a normal backup and the incremental backups will be updated to increment from the last normal backup.
